/*
 * p3pmail v1.0
 *
 * (C) 2004 by Jack S. Lai <laitcg@cox.net>
 *
 * It's intent is to parse dangerous html tags from email messages to make
 * them safe for viewing.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "p3pmail.h"

char *strreplace(char *buffer,char *search, char *replace){
   int searchlen  = strlen(search);
   int replacelen = strlen(replace);
   int bufferlen  = strlen(buffer);
   char * ptr;

   for (ptr=strstr(buffer,search); ptr!=0 ; ptr=strstr(ptr,search)){
      if (searchlen != replacelen) memmove( ptr+searchlen, ptr+replacelen, bufferlen - searchlen - (ptr-buffer) + 1);
      memcpy(ptr, replace, replacelen);
      ptr+=replacelen;
   }
   return buffer;
}

void parsehtml(char *buffer, int bufferlength){
   static char lastfew[FEWCHARACTERS + 1];
   static int firstline = 1;
   char *p;

   /* Do single line checks first */
   strreplace(buffer,"SRC","CRS");
   strreplace(buffer,"src","crs");
   strreplace(buffer,"HREF","FERH");
   strreplace(buffer,"href","ferh");

   /* Now look for broken lines by doing nothing for the first line,
      and something for subsequent. This will catch some broken lines.
   */
   if (firstline) {
      firstline = 0;
   } else if (strlen(lastfew) > 0 && (
      (strcasecmp(lastfew, "sr=") == 0 && strncasecmp(buffer, "c=3d", 4) == 0) ||
      (strcasecmp(lastfew, "s=") == 0 && strncasecmp(buffer, "rc=3d", 5) == 0) ||
      (strcasecmp(lastfew, "hre=") == 0 && strncasecmp(buffer, "f=3d", 4) == 0) ||
      (strcasecmp(lastfew, "hr=") == 0 && strncasecmp(buffer, "ef=3d", 5) == 0) ||
      (strcasecmp(lastfew, "h=") == 0 && strncasecmp(buffer, "ref=3d", 6) == 0))) {
      /* Ok, we have a match for HEX equals */
      buffer[0] = 'x'; // this is enough to scramble the attribute

   } else if (strlen(lastfew) > 0 && (
      (strcasecmp(lastfew, "sr=") == 0 && strncasecmp(buffer, "c=20", 4) == 0) ||
      (strcasecmp(lastfew, "s=") == 0 && strncasecmp(buffer, "rc=20", 5) == 0) ||
      (strcasecmp(lastfew, "hre=") == 0 && strncasecmp(buffer, "f=20", 4) == 0) ||
      (strcasecmp(lastfew, "hr=") == 0 && strncasecmp(buffer, "ef=20", 5) == 0) ||
      (strcasecmp(lastfew, "h=") == 0 && strncasecmp(buffer, "ref=20", 6) == 0))) {
      /* Ok, we have a match for HEX space */
      buffer[0] = 'x';
   }
    /*
       If there is a blank reasonably near the end of the line
       and if the last character is an "=" then we save the last
       few characters of current line for further processing with
       next line else we save an empty string
    */
    p = rindex(buffer, ' ');
    if (p && (bufferlength  - (p - buffer) - 1 <= FEWCHARACTERS) &&
        (buffer[bufferlength - 1] == '=')) {
        strcpy(lastfew, p + 1);
    } else {
        lastfew[0] = '\0';
    }
}

int main(){
   char line[MAX_BUF_SIZE];
   int pasthdr=0,html=0;

   while ((fgets(line,MAX_BUF_SIZE,stdin))!=NULL){
      /* Look for end of header*/
      if (!pasthdr){
         if ((*line == '\n') || (*line == '\r')){
            pasthdr=1;
            printf("X-HTML-Parser: %s %s by %s\n",PROGNAME,VERSION,AUTHORS);
         }
         printf("%s",line);
      }
      else {
         if (html){
            //printf("Pre : %s", line);
            parsehtml(line,strlen(line));
            /* Send output to stout (for capture) */
            printf("%s",line);
         }
         else {
/*            if ((strncasecmp(line,"<html>",6)==0 ||
               strncasecmp(line,"<head>",6)==0 ||
               strncasecmp(line,"<body>",6)==0 ||
               strncasecmp(line,"Content-Type: text/html",23)==0)){ */
            if ((strstr(line,"<html>")!=NULL ||
               strstr(line,"<head>")!=NULL ||
               strstr(line,"<body>")!=NULL ||
               strstr(line,"<HTML>")!=NULL ||
               strstr(line,"<HEAD>")!=NULL ||
               strstr(line,"<BODY>")!=NULL ||
               strncasecmp(line,"Content-Type: text/html",23)==0)){
                 html=1;
            }
            /* dont parse non-html*/
            printf("%s",line);
         }
         /* Keep skipping until past the header */
      }
   }
   /* All done, clean up */
   return 0;
}
